import os
import re
from sys import exit
import shutil
import logging
# ===GLOBAL=PRESETS=======================================
#os.chdir(os.path.dirname(sys.argv[0]))
logging.basicConfig(level=logging.DEBUG,
                    format='%(asctime)s %(levelname)-8s %(message)s',
                    datefmt='%m-%d %H:%M',
                    filename='log.txt',
                    filemode='w+')
console = logging.StreamHandler()
console.setLevel(logging.INFO)
formatter = logging.Formatter('%(levelname)-8s %(message)s')
console.setFormatter(formatter)
logging.getLogger('').addHandler(console)
sVer = "1.05 Py2Exe"
# ===HELPER=FUNCTIONS=====================================
def cleanLocalDir(dirN):
    if dirN == "Temp":
        logging.info("cleaning temp files")
    for f in os.listdir(dirN):
        fPath = os.path.join(dirN,f)
        try:
            if os.path.isfile(fPath):
                os.unlink(fPath)
        except Exception as e:
            print(e)

def verbExit():
    cleanLocalDir("Temp")
    logging.info("terminating script")
    input("Press Enter to continue...")
    exit()

def makeDirs(dirName):
    if not os.path.exists(dirName):
        os.makedirs(dirName, exist_ok=True)

def compFiles(f1,f2):
    with open(f1,"rb") as f1, open(f2,"rb") as f2:
        d1 = f1.read()
        d2 = f2.read()

    if len(d1) != len(d2):
        logging.error("wrong size after import!")
        verbExit()
    else:
        if d1 != d2:
            return False
        else:
            return True

def prepTempFiles(fIn,fOut):
    try:
        shutil.copy(fIn,"Temp/workfile.from")
    except FileNotFoundError:
        logging.error("File not found: %s" %fIn)
        verbExit()
    try:
        shutil.copy(fOut,"Temp/workfile.to")
    except FileNotFoundError:
        logging.error("File not found: %s" %fOut)
        verbExit()

    if os.path.isfile("Temp/workfile.from"):
        logging.info("sourceFile copied successfully")
    else:
        logging.error("could not copy source file")
        verbExit()
    if os.path.isfile("Temp/workfile.to"):
        logging.info("destFile copied successfully")
    else:
        logging.error("could not copy destination file")
        verbExit()


def seekPattern(file):
    with open(file,"rb") as f:
        indexes = []
        data = f.read()
        pattern = b'\x6E\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x3F'
        logging.debug("patterns found: %i" %data.count(pattern))
        # for re. \x3F is always translated to ? and treated as regex symbol although it shouldn't with b' flag :/
        # workaround is to replace it as a proper char and then escape it
        pattern = re.compile(b'\x6E\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\?')
        for p in re.finditer(pattern,data):
            logging.debug("%i:%i [%s:%s]"  %(p.start(),p.end(),hex(p.start()),hex(p.end())))
            indexes.extend([p.start()])

    return indexes

def getCharBlock(file,index):
    logging.debug("trying to open %s" % file)
    with open(file,"rb") as f:
        f.seek(index)
        charBlock = f.read(8804)
    return charBlock

def writeCharBlock(file,index,charBlock):
    with open(file,"r+b") as f:
        f.seek(index)
        f.write(charBlock)

def printIntro():
    print("================================")
    print("BOSS IMPORT script for SR3/4 -> GOOH v.%s" %sVer)
    print("please report any problems in this thread:")
    print('https://www.saintsrowmods.com/forum/threads/import-boss-from-sr4-to-gooh.14763/')
    print("================================")
    print("!!ALWAYS BACKUP YOUR SAVES AND USE AT YOUR OWN RISK!!")
    print("================================")
    print("")

# ===MAIN=BLOCK===========================================
def mainRun():
    newF = False
    makeDirs("Temp")
    makeDirs("Out")
    cleanLocalDir("Out")

    fIn = input("Please provide a full path to SR3/SR4 save you want to import FROM:\n")
    print("-----------------")
    fOut = input("Please provide a full path to GOOH save you want to import TO\n(leave empty for a newgame save to be created):\n")
    logging.debug("path FROM: %s" %fIn)

    if fOut == "":
        newF = True
        fOut = "./srgsave_00.srgs_pcDEFAULT"
        logging.debug("path TO: NEWFILE")
    else:
        logging.debug("path TO: %s" %fOut)
    print("-----------------")
    prepTempFiles(fIn,fOut)

    workIn = "Temp/workfile.from"
    workOut = "Temp/workfile.to"

    iIn = seekPattern(workIn)
    if len(iIn) != 1:
        logging.error("%i patterns found!\nfile not valid for import" %len(iIn))
        verbExit()
    else:
        logging.info("valid source file")

    iOut = seekPattern(workOut)
    if len(iOut) != 2:
        logging.warning("Boss record not found in GOOH file. Attempting to write at hardcoded address")
        iOut = 32676
    else:
        logging.info("Boss record found. It will be overwritten with the imported one")
        iOut = iOut[1]

    logging.debug("index in source file: %i" %iIn[0])
    logging.debug("index in dest file: %i" %iOut)

    bossBlock = getCharBlock(workIn,iIn[0])
    logging.debug("charBlock length: %i" %len(bossBlock))
    try:
        writeCharBlock(workOut,iOut,bossBlock)
    except Exception as e:
        logging.error(e)
        verbExit()
    if compFiles(fOut,workOut):
        logging.error("file unchanged. Import unsuccessful.")
        verbExit()
    else:
        if newF:
            shutil.copy("./AUTOSAVE_ONLY_savedir.srgd_pc","./Out/savedir.srgd_pc")
        shutil.copy(workOut,"./Out/srgsave_00.srgs_pc")
        if os.path.isfile("./Out/srgsave_00.srgs_pc"):
            logging.info("Modified file ready at .\\Out\\srgsave_00.srgs_pc")
            logging.warning("!!REMEMBER TO BACKUP YOUR ORIGINAL SAVES BEFORE REPLACING THEM WITH MODIFIED ONES!!")
            verbExit()
        else:
            logging.error("Could not copy tempfile!")
            logging.warning("script will now terminate without cleaning Temp folder.\nTry renaming .\\Temp\\workfile.to file to 'srgsave_00.srgs_pc' and loading in-game")
            exit()

logging.info("Logging to ./log.txt. See this file for more detailed info.")
printIntro()
mainRun()

# ===DARKLANDS=BELOW=DO=NOT=PROCEED=======================
# color test
# def print_format_table():
#     """
#     prints table of formatted text format options
#     """
#     for style in range(8):
#         for fg in range(30,38):
#             s1 = ''
#             for bg in range(40,48):
#                 format = ';'.join([str(style), str(fg), str(bg)])
#                 s1 += '\x1b[%sm %s \x1b[0m' % (format, format)
#             print(s1)
#         print('\n')
#
# print_format_table()